@@ -9,34 +9,26 @@
 #   ./build-utils/getversion.sh export-subst
 #
 # This tells git to replace the format string in the following line with the
-# current hash upon the calling of the `git archive <hash/tag>` command.
-VERSION_FROM_ARCHIVE=636637cdbe734c615dbbf2fd9c5533295d0a79c1
+# current short hash upon the calling of the `git archive <hash/tag>` command.
+VERSION_FROM_ARCHIVE=b1fb7fa
 
 # The preferred method is to use the git describe command but this is only
 # possible if the .git directory is present.
 if [ -d .git -a -r .git ]
 then
-    VERSION_FROM_GIT=`git describe --tags --always`
-fi
-
-# In case of a worktree checkout, there is not .git directory, but a
-# .git file, which contains the path to the branch information within
-# the git repository. In this case, "git describe" works too.
-if [ -f .git -a -r .git ] && grep -q ^gitdir .git
-then
-    VERSION_FROM_GIT=`git describe --tags --always`
+    VERSION_FROM_GIT=$(git describe --tags --always)
 fi
 
 if [ x"$VERSION_FROM_GIT" != x ]; then
     echo $VERSION_FROM_GIT; exit 0;
 fi
 
-if [ "$VERSION_FROM_ARCHIVE" != ':%H$' ]; then
-    echo $VERSION_FROM_ARCHIVE | cut -b1-8; exit 0;
+if [ "$VERSION_FROM_ARCHIVE" != ':%h$' ]; then
+    echo $VERSION_FROM_ARCHIVE; exit 0;
 fi
 
-echo "ERROR: Git commit hash detection failed: Please check why "
-     "build-utils/getversion.sh is failing in your setup and get in touch with us." >&2
+echo "ERROR: Commit hash detection failure. Dear packager, please figure out"\
+     "what has gone wrong and or get in touch with us." >&2
 
 exit 2
 
