@@ -28,16 +28,10 @@ typedef struct widget_t widget_t;
 #include "common/luaclass.h"
 #include "common/luaobject.h"
 #include "luah.h"
-#include "globalconf.h"
 
 #include <gtk/gtk.h>
 
-#define GOBJECT_LUAKIT_WIDGET_DATA_KEY "luakit_widget_data"
-
-#define GOBJECT_TO_LUAKIT_WIDGET(gtk_widget) ((widget_t*)g_object_get_data(G_OBJECT(gtk_widget), \
-            GOBJECT_LUAKIT_WIDGET_DATA_KEY))
-
-typedef widget_t *(widget_constructor_t)(lua_State *L, widget_t *, luakit_token_t);
+typedef widget_t *(widget_constructor_t)(widget_t *, luakit_token_t);
 typedef void (widget_destructor_t)(widget_t *);
 
 widget_constructor_t widget_box;
@@ -48,12 +42,7 @@ widget_constructor_t widget_notebook;
 widget_constructor_t widget_paned;
 widget_constructor_t widget_webview;
 widget_constructor_t widget_window;
-widget_constructor_t widget_overlay;
-widget_constructor_t widget_scrolled;
-widget_constructor_t widget_image;
-widget_constructor_t widget_spinner;
-widget_constructor_t widget_drawing_area;
-widget_constructor_t widget_stack;
+widget_constructor_t widget_socket;
 
 typedef const struct {
     luakit_token_t tok;
@@ -70,35 +59,26 @@ struct widget_t
     /* Widget destructor */
     widget_destructor_t *destructor;
     /* Index function */
-    gint (*index)(lua_State *, widget_t *, luakit_token_t);
+    gint (*index)(lua_State *, luakit_token_t);
     /* Newindex function */
-    gint (*newindex)(lua_State *, widget_t *, luakit_token_t);
+    gint (*newindex)(lua_State *, luakit_token_t);
     /* Lua object ref */
     gpointer ref;
     /* Main gtk widget */
     GtkWidget *widget;
-#if GTK_CHECK_VERSION(3,16,0)
-    /* CSS provider for this widget */
-    GtkCssProvider *provider;
-#endif
-    /* Previous width and height, for resize signal */
-    gint prev_width, prev_height;
     /* Misc private data */
     gpointer data;
 };
 
-extern lua_class_t widget_class;
+lua_class_t widget_class;
 void widget_class_setup(lua_State *);
-void widget_set_css_properties(widget_t *, ...);
-gint luaH_widget_new(lua_State *L);
 
 static inline widget_t*
 luaH_checkwidget(lua_State *L, gint udx)
 {
     widget_t *w = luaH_checkudata(L, udx, &widget_class);
     if (!w->widget)
-        luaL_error(L, "widget %p (%s) has been destroyed", w, w->info->name);
-    g_assert(GTK_IS_WIDGET(w->widget));
+        luaL_argerror(L, udx, "using destroyed widget");
     return w;
 }
 
