@@ -1,5 +1,5 @@
 /*
- * common/luaobject.c - useful functions for handling Lua objects
+ * luaobject.c - useful functions for handling Lua objects
  *
  * Copyright © 2010 Mason Larobina <mason.larobina@gmail.com>
  * Copyright © 2009 Julien Danjou <julien@danjou.info>
@@ -148,17 +148,6 @@ luaH_object_add_signal(lua_State *L, gint oud,
         const gchar *name, gint ud) {
     luaH_checkfunction(L, ud);
     lua_object_t *obj = lua_touserdata(L, oud);
-    if (!obj) {
-        warn("object add signal on non object");
-        return;
-    }
-
-    gchar *origin = luaH_callerinfo(L);
-    debug("add " ANSI_COLOR_BLUE "\"%s\"" ANSI_COLOR_RESET
-            " on %p from " ANSI_COLOR_GREEN "%s" ANSI_COLOR_RESET,
-            name, obj, origin);
-    g_free(origin);
-
     signal_add(obj->signals, name, luaH_object_ref_item(L, oud, ud));
 }
 
@@ -172,66 +161,45 @@ luaH_object_remove_signal(lua_State *L, gint oud,
         const gchar *name, gint ud) {
     luaH_checkfunction(L, ud);
     lua_object_t *obj = lua_touserdata(L, oud);
-    if (!obj) {
-        warn("object remove signal on non object");
-        return;
-    }
     gpointer ref = (gpointer) lua_topointer(L, ud);
     signal_remove(obj->signals, name, ref);
     luaH_object_unref_item(L, oud, ref);
     lua_remove(L, ud);
 }
 
-/* Remove all signals of a given name to an object.
- * `oud` is the object index on the stack.
+/* Emit a signal from a signals array and return the results of the first
+ * handler that returns something.
+ * `signals` is the signals array.
  * `name` is the name of the signal.
- */
-void
-luaH_object_remove_signals(lua_State *L, gint oud, const gchar *name) {
-    lua_object_t *obj = lua_touserdata(L, oud);
-    if (!obj) {
-        warn("object remove signals on non object");
-        return;
-    }
-    signal_array_t *sigfuncs = signal_lookup(obj->signals, name);
-    if (!sigfuncs)
-        return;
-    for (guint i = 0; i < sigfuncs->len; i++) {
-        gpointer ref = g_ptr_array_index(sigfuncs, i);
-        luaH_object_unref_item(L, oud, ref);
-    }
-    signals_remove(obj->signals, name);
-}
-
-/* Similar to signal_object_emit(), but allows you to choose the signal array
- * by name. */
+ * `nargs` is the number of arguments to pass to the called functions.
+ * `nret` is the number of return values this function pushes onto the stack.
+ * A positive number means that any missing values will be padded with nil
+ * and any superfluous values will be removed.
+ * LUA_MULTRET means that any number of values is returned without any
+ * adjustment.
+ * 0 means that all return values are removed and that ALL handler functions are
+ * executed.
+ * Returns the number of return values pushed onto the stack. */
 gint
-signal_array_emit(lua_State *L, signal_t *signals,
-        const gchar *array_name, const gchar *name, gint nargs, gint nret) {
+signal_object_emit(lua_State *L, signal_t *signals,
+        const gchar *name, gint nargs, gint nret) {
 
-    signal_array_t *sigfuncs = signal_lookup(signals, array_name);
-
-    gchar *origin = luaH_callerinfo(L);
-    debug("emit " ANSI_COLOR_BLUE "\"%s\"" ANSI_COLOR_RESET
-            " on %p from "
-            ANSI_COLOR_GREEN "%s" ANSI_COLOR_RESET " (%d args, %d nret)",
-            name, signals, origin ? origin : "<GTK>", nargs, nret);
-    g_free(origin);
-
-    if (sigfuncs) {
+    signal_array_t *sigfuncs = signal_lookup(signals, name);
+    debug("emitting \"%s\" with %d args and %d nret", name, nargs, nret);
+    if(sigfuncs) {
         gint nbfunc = sigfuncs->len;
         luaL_checkstack(L, lua_gettop(L) + nbfunc + nargs + 1,
-                "too many signal handlers; need a new implementation!");
+                "too much signal");
         /* Push all functions and then execute, because this list can change
          * while executing funcs. */
-        for (gint i = 0; i < nbfunc; i++) {
+        for(gint i = 0; i < nbfunc; i++) {
             luaH_object_push(L, sigfuncs->pdata[i]);
         }
 
-        for (gint i = 0; i < nbfunc; i++) {
+        for(gint i = 0; i < nbfunc; i++) {
             gint stacksize = lua_gettop(L);
             /* push all args */
-            for (gint j = 0; j < nargs; j++)
+            for(gint j = 0; j < nargs; j++)
                 lua_pushvalue(L, - nargs - nbfunc + i);
             /* push first function */
             lua_pushvalue(L, - nargs - nbfunc + i);
@@ -240,12 +208,8 @@ signal_array_emit(lua_State *L, signal_t *signals,
             luaH_dofunction(L, nargs, LUA_MULTRET);
             gint ret = lua_gettop(L) - stacksize + 1;
 
-            /* Signal execution stops when:
-             *  - there's an expected number of return values (>0 or LUA_MULTRET)
-             *  - at least one return value (ret)
-             *  - the first return value is non-nil
-             */
-            if (nret && ret && !lua_isnil(L, -ret)) {
+            /* Note that only if nret && ret will the signal execution stop */
+            if (nret && ret) {
                 /* remove all args and functions */
                 for (gint j = 0; j < nargs + nbfunc - i - 1; j++) {
                     lua_remove(L, - ret - 1);
@@ -276,26 +240,6 @@ signal_array_emit(lua_State *L, signal_t *signals,
     return 0;
 }
 
-/* Emit a signal from a signals array and return the results of the first
- * handler that returns something non-nil.
- * `signals` is the signals array.
- * `name` is the name of the signal.
- * `nargs` is the number of arguments to pass to the called functions.
- * `nret` is the number of return values this function pushes onto the stack.
- * A positive number means that any missing values will be padded with nil
- * and any superfluous values will be removed.
- * LUA_MULTRET means that any number of values is returned without any
- * adjustment.
- * 0 means that all return values are removed and that ALL handler functions are
- * executed.
- * Returns the number of return values pushed onto the stack. */
-gint
-signal_object_emit(lua_State *L, signal_t *signals,
-        const gchar *name, gint nargs, gint nret) {
-
-    return signal_array_emit(L, signals, name, name, nargs, nret);
-}
-
 /* Emit a signal to an object.
  * `oud` is the object index on the stack.
  * `name` is the name of the signal.
@@ -314,34 +258,23 @@ luaH_object_emit_signal(lua_State *L, gint oud,
     gint ret, top, bot = lua_gettop(L) - nargs + 1;
     gint oud_abs = luaH_absindex(L, oud);
     lua_object_t *obj = lua_touserdata(L, oud);
-    if (!obj)
-        return luaL_error(L, "trying to emit " ANSI_COLOR_BLUE "\"%s\"" ANSI_COLOR_RESET " on non-object", name);
-
-    gchar *origin = luaH_callerinfo(L);
-    debug("emit " ANSI_COLOR_BLUE "\"%s\"" ANSI_COLOR_RESET
-            " on %p from "
-            ANSI_COLOR_GREEN "%s" ANSI_COLOR_RESET " (%d args, %d nret)",
-            name, obj, origin ? origin : "<GTK>", nargs, nret);
-    g_free(origin);
-
+    debug("emitting \"%s\" on %p with %d args and %d nret", name, obj, nargs, nret);
     if(!obj)
-        return luaL_error(L, "trying to emit " ANSI_COLOR_BLUE "\"%s\"" ANSI_COLOR_RESET " on non-object", name);
-
+        luaL_error(L, "trying to emit signal on non-object");
     signal_array_t *sigfuncs = signal_lookup(obj->signals, name);
-    if (sigfuncs) {
+    if(sigfuncs) {
         guint nbfunc = sigfuncs->len;
-        luaL_checkstack(L, lua_gettop(L) + nbfunc + nargs + 2,
-                "too many signal handlers; need a new implementation!");
+        luaL_checkstack(L, lua_gettop(L) + nbfunc + nargs + 2, "too much signal");
         /* Push all functions and then execute, because this list can change
          * while executing funcs. */
-        for (guint i = 0; i < nbfunc; i++)
+        for(guint i = 0; i < nbfunc; i++)
             luaH_object_push_item(L, oud_abs, sigfuncs->pdata[i]);
 
-        for (guint i = 0; i < nbfunc; i++) {
+        for(guint i = 0; i < nbfunc; i++) {
             /* push object */
             lua_pushvalue(L, oud_abs);
             /* push all args */
-            for (gint j = 0; j < nargs; j++)
+            for(gint j = 0; j < nargs; j++)
                 lua_pushvalue(L, - nargs - nbfunc - 1 + i);
             /* push first function */
             lua_pushvalue(L, - nargs - nbfunc - 1 + i);
@@ -351,12 +284,8 @@ luaH_object_emit_signal(lua_State *L, gint oud,
             luaH_dofunction(L, nargs + 1, LUA_MULTRET);
             ret = lua_gettop(L) - top;
 
-            /* Signal execution stops when:
-             *  - there's an expected number of return values (>0 or LUA_MULTRET)
-             *  - at least one return value (ret)
-             *  - the first return value is non-nil
-             */
-            if (nret && ret && !lua_isnil(L, -ret)) {
+            /* Note that only if nret && ret will the signal execution stop */
+            if (nret && ret) {
                 /* Adjust the number of results to match nret (including 0) */
                 if (nret != LUA_MULTRET && ret != nret) {
                     /* Pad with nils */
@@ -383,15 +312,6 @@ luaH_object_emit_signal(lua_State *L, gint oud,
     return 0;
 }
 
-gint
-luaH_object_property_signal(lua_State *L, gint oud, luakit_token_t tok)
-{
-    gchar *signame = g_strdup_printf("property::%s", token_tostring(tok));
-    luaH_object_emit_signal(L, oud, signame, 0, 0);
-    g_free(signame);
-    return 0;
-}
-
 gint
 luaH_object_add_signal_simple(lua_State *L) {
     luaH_object_add_signal(L, 1, luaL_checkstring(L, 2), 3);
@@ -404,36 +324,6 @@ luaH_object_remove_signal_simple(lua_State *L) {
     return 0;
 }
 
-gint
-luaH_object_remove_signals_simple(lua_State *L) {
-    luaH_object_remove_signals(L, 1, luaL_checkstring(L, 2));
-    return 0;
-}
-
-gboolean
-luaH_object_collect_signal_keys(gpointer key, gpointer UNUSED(value), GPtrArray *keys)
-{
-    g_ptr_array_add(keys, key);
-    return FALSE;
-}
-
-gint
-luaH_object_remove_all_signals(signal_t *signals)
-{
-    if (signals) {
-        lua_State *L = common.L;
-        GPtrArray *keys = g_ptr_array_new();
-        g_tree_foreach(signals, (GTraverseFunc)luaH_object_collect_signal_keys, keys);
-        for (guint i = 0; i < keys->len; i++) {
-            char *type = g_ptr_array_index(keys, i);
-            lua_pushstring(L, type);
-            luaH_object_remove_signals_simple(L);
-        }
-        g_ptr_array_free(keys, FALSE);
-    }
-    return 0;
-}
-
 gint
 luaH_object_emit_signal_simple(lua_State *L) {
     return luaH_object_emit_signal(L, 1, luaL_checkstring(L, 2), lua_gettop(L) - 2, LUA_MULTRET);
@@ -453,14 +343,8 @@ luaH_object_tostring(lua_State *L) {
 gint
 luaH_object_gc(lua_State *L) {
     lua_object_t *item = lua_touserdata(L, 1);
-    if (!item) {
-        warn("garbage collect on non-object");
-        return 0;
-    }
-    if (item->signals) {
-        luaH_object_remove_all_signals(item->signals);
+    if (item->signals)
         signal_destroy(item->signals);
-    }
     return 0;
 }
 
