@@ -22,38 +22,42 @@
 #define LUAKIT_COMMON_PROPERTY_H
 
 #include <lua.h>
-#include <glib-object.h>
-
-#include "common/tokenize.h"
+#include <glib/ghash.h>
 
 typedef enum {
     BOOL,
     CHAR,
-    DOUBLE,
-    FLOAT,
     INT,
+    FLOAT,
+    DOUBLE,
     URI,
 } property_value_t;
 
+typedef enum {
+    SETTINGS,
+    WEBKITVIEW,
+    SESSION,
+    COOKIEJAR,
+} property_scope;
+
 typedef union {
     gchar *c;
     gboolean b;
     gdouble d;
     gfloat f;
     gint i;
-} property_tmp_t;
+} property_tmp_value_t;
 
 typedef struct {
-    luakit_token_t tok;
     const gchar *name;
     property_value_t type;
+    property_scope scope;
     gboolean writable;
+    const gchar *signame;
 } property_t;
 
-gint luaH_gobject_index(lua_State *, property_t *, luakit_token_t, GObject *);
-gboolean luaH_gobject_newindex(lua_State *, property_t *, luakit_token_t,
-        gint, GObject *);
+GHashTable* hash_properties(property_t *properties);
+gint luaH_get_property(lua_State *L, GHashTable *properties, gpointer obj, gint nidx);
+gint luaH_set_property(lua_State *L, GHashTable *properties, gpointer obj, gint nidx, gint vidx);
 
 #endif
-
-// vim: ft=c:et:sw=4:ts=8:sts=4:tw=80
