@@ -21,21 +21,30 @@
 #include "luah.h"
 #include "widgets/common.h"
 
+static gint
+luaH_entry_append(lua_State *L)
+{
+    size_t len;
+    widget_t *w = luaH_checkwidget(L, 1);
+    const gchar *text = luaL_checklstring(L, 2, &len);
+    gint pos = -1;
+    gtk_editable_insert_text(GTK_EDITABLE(w->widget),
+        text, g_utf8_strlen(text, len), &pos);
+    lua_pushnumber(L, pos);
+    return 1;
+}
+
 static gint
 luaH_entry_insert(lua_State *L)
 {
+    size_t len;
     widget_t *w = luaH_checkwidget(L, 1);
-
-    /* get insert position (or append text) */
-    gint pos = -1, idx = 2;
-    if (lua_gettop(L) > 2) {
-        pos = luaL_checknumber(L, idx++);
-        if (pos > 0) pos--; /* correct lua index */
-    }
-
+    gint pos = luaL_checknumber(L, 2);
+    const gchar *text = luaL_checklstring(L, 3, &len);
     gtk_editable_insert_text(GTK_EDITABLE(w->widget),
-        luaL_checkstring(L, idx), -1, &pos);
-    return 0;
+        text, g_utf8_strlen(text, len), &pos);
+    lua_pushnumber(L, pos);
+    return 1;
 }
 
 static gint
@@ -52,12 +61,16 @@ luaH_entry_select_region(lua_State* L)
 }
 
 static gint
-luaH_entry_index(lua_State *L, widget_t *w, luakit_token_t token)
+luaH_entry_index(lua_State *L, luakit_token_t token)
 {
-    switch(token) {
-      LUAKIT_WIDGET_INDEX_COMMON(w)
+    widget_t *w = luaH_checkwidget(L, 1);
+
+    switch(token)
+    {
+      LUAKIT_WIDGET_INDEX_COMMON
 
       /* push class methods */
+      PF_CASE(APPEND,           luaH_entry_append)
       PF_CASE(INSERT,           luaH_entry_insert)
       PF_CASE(SELECT_REGION,    luaH_entry_select_region)
       /* push integer properties */
@@ -77,18 +90,16 @@ luaH_entry_index(lua_State *L, widget_t *w, luakit_token_t token)
 }
 
 static gint
-luaH_entry_newindex(lua_State *L, widget_t *w, luakit_token_t token)
+luaH_entry_newindex(lua_State *L, luakit_token_t token)
 {
     size_t len;
+    widget_t *w = luaH_checkwidget(L, 1);
     const gchar *tmp;
-    GdkRGBA c;
-#if !GTK_CHECK_VERSION(3,16,0)
+    GdkColor c;
     PangoFontDescription *font;
-#endif
-
-    switch(token) {
-      LUAKIT_WIDGET_NEWINDEX_COMMON(w)
 
+    switch(token)
+    {
       case L_TK_TEXT:
         gtk_entry_set_text(GTK_ENTRY(w->widget),
             luaL_checklstring(L, 3, &len));
@@ -97,22 +108,13 @@ luaH_entry_newindex(lua_State *L, widget_t *w, luakit_token_t token)
       case L_TK_FG:
       case L_TK_BG:
         tmp = luaL_checklstring(L, 3, &len);
-        if (!gdk_rgba_parse(&c, tmp))
+        if (!gdk_color_parse(tmp, &c))
             luaL_argerror(L, 3, "unable to parse color");
         if (token == L_TK_FG) {
-#if GTK_CHECK_VERSION(3,16,0)
-            widget_set_css_properties(w, "color", tmp, NULL);
-            widget_set_css_properties(w, "caret-color", tmp, NULL);
-#else
-            gtk_widget_override_color(GTK_WIDGET(w->widget), GTK_STATE_FLAG_NORMAL, &c);
-#endif
+            gtk_widget_modify_text(GTK_WIDGET(w->widget), GTK_STATE_NORMAL, &c);
             g_object_set_data_full(G_OBJECT(w->widget), "fg", g_strdup(tmp), g_free);
         } else {
-#if GTK_CHECK_VERSION(3,16,0)
-            widget_set_css_properties(w, "background-color", tmp, NULL);
-#else
-            gtk_widget_override_background_color(GTK_WIDGET(w->widget), GTK_STATE_FLAG_NORMAL, &c);
-#endif
+            gtk_widget_modify_base(GTK_WIDGET(w->widget), GTK_STATE_NORMAL, &c);
             g_object_set_data_full(G_OBJECT(w->widget), "bg", g_strdup(tmp), g_free);
         }
         break;
@@ -127,26 +129,23 @@ luaH_entry_newindex(lua_State *L, widget_t *w, luakit_token_t token)
 
       case L_TK_FONT:
         tmp = luaL_checklstring(L, 3, &len);
-#if GTK_CHECK_VERSION(3,16,0)
-        widget_set_css_properties(w, "font", tmp, NULL);
-#else
         font = pango_font_description_from_string(tmp);
-        gtk_widget_override_font(GTK_WIDGET(w->widget), font);
-#endif
+        gtk_widget_modify_font(GTK_WIDGET(w->widget), font);
         g_object_set_data_full(G_OBJECT(w->widget), "font", g_strdup(tmp), g_free);
         break;
 
       default:
-        luaH_warn(L, "unknown property: %s", luaL_checkstring(L, 2));
+        warn("unknown property: %s", luaL_checkstring(L, 2));
         return 0;
     }
-    return luaH_object_property_signal(L, 1, token);
+    return luaH_object_emit_property_signal(L, 1);
 }
 
 static void
-activate_cb(GtkEntry* UNUSED(e), widget_t *w)
+activate_cb(GtkEntry *e, widget_t *w)
 {
-    lua_State *L = common.L;
+    (void) e;
+    lua_State *L = globalconf.L;
     luaH_object_push(L, w->ref);
     luaH_object_emit_signal(L, -1, "activate", 0, 0);
     lua_pop(L, 1);
@@ -155,53 +154,57 @@ activate_cb(GtkEntry* UNUSED(e), widget_t *w)
 static void
 changed_cb(widget_t *w)
 {
-    lua_State *L = common.L;
+    lua_State *L = globalconf.L;
     luaH_object_push(L, w->ref);
     luaH_object_emit_signal(L, -1, "changed", 0, 0);
     lua_pop(L, 1);
 }
 
 static void
-position_cb(GtkEntry* UNUSED(e), GParamSpec* UNUSED(ps), widget_t *w)
+position_cb(GtkEntry *e, GParamSpec *ps, widget_t *w)
 {
-    lua_State *L = common.L;
+    (void) e;
+    (void) ps;
+    lua_State *L = globalconf.L;
     luaH_object_push(L, w->ref);
     luaH_object_emit_signal(L, -1, "property::position", 0, 0);
     lua_pop(L, 1);
 }
 
 widget_t *
-widget_entry(lua_State *UNUSED(L), widget_t *w, luakit_token_t UNUSED(token))
+widget_entry(widget_t *w)
 {
     w->index = luaH_entry_index;
     w->newindex = luaH_entry_newindex;
+    w->destructor = widget_destructor;
 
     /* create gtk label widget as main widget */
     w->widget = gtk_entry_new();
+    g_object_set_data(G_OBJECT(w->widget), "lua_widget", (gpointer) w);
 
     /* setup default settings */
-#if GTK_CHECK_VERSION(3,4,0)
-    GtkStyleContext *context = gtk_widget_get_style_context(GTK_WIDGET(w->widget));
-    const gchar *inputbar_css = "GtkEntry {border: none; padding: 2px;}";
-
-    GtkCssProvider *provider = gtk_css_provider_new();
-    gtk_css_provider_load_from_data(provider, inputbar_css, strlen(inputbar_css), NULL);
-
-    gtk_style_context_add_provider(context, GTK_STYLE_PROVIDER(provider), GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
-#else
     gtk_entry_set_inner_border(GTK_ENTRY(w->widget), NULL);
-#endif
 
     g_object_connect(G_OBJECT(w->widget),
-      LUAKIT_WIDGET_SIGNAL_COMMON(w)
       "signal::activate",                          G_CALLBACK(activate_cb),   w,
+      "signal::focus-in-event",                    G_CALLBACK(focus_cb),      w,
+      "signal::focus-out-event",                   G_CALLBACK(focus_cb),      w,
       "signal::key-press-event",                   G_CALLBACK(key_press_cb),  w,
+      "signal::parent-set",                        G_CALLBACK(parent_set_cb), w,
       "signal::notify::cursor-position",           G_CALLBACK(position_cb),   w,
+      // The following signals replace the old "signal::changed", since that
+      // does not allow for the selection to be changed in it's callback.
+      "swapped-signal-after::backspace",           G_CALLBACK(changed_cb),    w,
+      "swapped-signal-after::delete-from-cursor",  G_CALLBACK(changed_cb),    w,
+      "swapped-signal-after::insert-at-cursor",    G_CALLBACK(changed_cb),    w,
+      "swapped-signal-after::paste-clipboard",     G_CALLBACK(changed_cb),    w,
+      "swapped-signal::button-release-event",      G_CALLBACK(changed_cb),    w,
       NULL);
 
     // Further signal to replace "signal::changed"
-    g_object_connect(G_OBJECT(w->widget),
-      "swapped-signal::changed", G_CALLBACK(changed_cb), w,
+    GtkEntry* entry = GTK_ENTRY(w->widget);
+    g_object_connect(G_OBJECT(entry->im_context),
+      "swapped-signal::commit", G_CALLBACK(changed_cb), w,
       NULL);
 
     gtk_widget_show(w->widget);
